/**
 * Copyright (C) 2020 Wasabeef
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * See about document.execCommand: https://developer.mozilla.org/en-US/docs/Web/API/Document/execCommand
 */

var RE = {};

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0};

RE.editor = document.getElementById('editor');

document.addEventListener("selectionchange", function() { RE.backuprange(); });

// Initializations
RE.callback = function() {
    window.location.href = "re-callback://" + encodeURIComponent(RE.getHtml());
}

RE.setHtml = function(contents) {
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
}

RE.getHtml = function() {
    return RE.editor.innerHTML;
}

RE.getText = function() {
    return RE.editor.innerText;
}

RE.setBaseTextColor = function(color) {
    RE.editor.style.color  = color;
}

RE.setBaseFontSize = function(size) {
    RE.editor.style.fontSize = size;
}

RE.setPadding = function(left, top, right, bottom) {
  RE.editor.style.paddingLeft = left;
  RE.editor.style.paddingTop = top;
  RE.editor.style.paddingRight = right;
  RE.editor.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function(color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function(image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
}

RE.setHeight = function(size) {
    RE.editor.style.height = size;
}

RE.setTextAlign = function(align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function(align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

RE.setInputEnabled = function(inputEnabled) {
    RE.editor.contentEditable = String(inputEnabled);
}

RE.undo = function() {
    document.execCommand('undo', false, null);
}

RE.redo = function() {
    document.execCommand('redo', false, null);
}

RE.setBold = function() {
    document.execCommand('bold', false, null);
}

RE.setItalic = function() {
    document.execCommand('italic', false, null);
}

RE.setSubscript = function() {
    document.execCommand('subscript', false, null);
}

RE.setSuperscript = function() {
    document.execCommand('superscript', false, null);
}

RE.setStrikeThrough = function() {
    document.execCommand('strikeThrough', false, null);
}

RE.setUnderline = function() {
    document.execCommand('underline', false, null);
}

RE.setBullets = function() {
    document.execCommand('insertUnorderedList', false, null);
}

RE.setNumbers = function() {
    document.execCommand('insertOrderedList', false, null);
}

RE.setTextColor = function(color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setTextBackgroundColor = function(color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('hiliteColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setFontSize = function(fontSize){
    document.execCommand("fontSize", false, fontSize);
}

RE.setHeading = function(heading) {
    document.execCommand('formatBlock', false, '<h'+heading+'>');
}

RE.setIndent = function() {
    document.execCommand('indent', false, null);
}

RE.setOutdent = function() {
    document.execCommand('outdent', false, null);
}

RE.setJustifyLeft = function() {
    document.execCommand('justifyLeft', false, null);
}

RE.setJustifyCenter = function() {
    document.execCommand('justifyCenter', false, null);
}

RE.setJustifyRight = function() {
    document.execCommand('justifyRight', false, null);
}

RE.setBlockquote = function() {
    document.execCommand('formatBlock', false, '<blockquote>');
}

RE.insertImage = function(url, alt) {
    var html = '<img src="' + url + '" alt="' + alt + '" />';
    RE.insertHTML(html);
}

RE.insertImageW = function(url, alt, width) {
    var html = '<img src="' + url + '" alt="' + alt + '" width="' + width + '"/>';
    RE.insertHTML(html);
}

RE.insertImageWH = function(url, alt, width, height) {
    var html = '<img src="' + url + '" alt="' + alt + '" width="' + width + '" height="' + height +'"/>';
    RE.insertHTML(html);
}

RE.insertVideo = function(url, alt) {
    var html = '<video src="' + url + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertVideoW = function(url, width) {
    var html = '<video src="' + url + '" width="' + width + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertVideoWH = function(url, width, height) {
    var html = '<video src="' + url + '" width="' + width + '" height="' + height + '" controls></video><br>';
    RE.insertHTML(html);
}

RE.insertAudio = function(url, alt) {
    var html = '<audio src="' + url + '" controls></audio><br>';
    RE.insertHTML(html);
}

RE.insertYoutubeVideo = function(url) {
    var html = '<iframe width="100%" height="100%" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertYoutubeVideoW = function(url, width) {
    var html = '<iframe width="' + width + '" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertYoutubeVideoWH = function(url, width, height) {
    var html = '<iframe width="' + width + '" height="' + height + '" src="' + url + '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    RE.insertHTML(html);
}

RE.insertHTML = function(html) {
    RE.restorerange();
    document.execCommand('insertHTML', false, html);
}

RE.insertLink = function(url, title) {
    RE.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        document.execCommand("insertHTML",false,"<a href='"+url+"'>"+title+"</a>");
    } else if (sel.rangeCount) {
       var el = document.createElement("a");
       el.setAttribute("href", url);
       el.setAttribute("title", title);

       var range = sel.getRangeAt(0).cloneRange();
       range.surroundContents(el);
       sel.removeAllRanges();
       sel.addRange(range);
   }
    RE.callback();
}



RE.prepareInsert = function() {
    RE.backuprange();
}

RE.backuprange = function(){
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
      var range = selection.getRangeAt(0);
      RE.currentSelection = {
          "startContainer": range.startContainer,
          "startOffset": range.startOffset,
          "endContainer": range.endContainer,
          "endOffset": range.endOffset};
    }
}

RE.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}

RE.enabledEditingItems = function(e) {
    var items = [];
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }

    window.location.href = "re-state://" + encodeURI(items.join(','));
}

RE.focus = function() {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    RE.editor.focus();
}

RE.blurFocus = function() {
    RE.editor.blur();
}

RE.removeFormat = function() {
    document.execCommand('removeFormat', false, null);
}
// Event Listeners
RE.editor.addEventListener("input", RE.callback);

RE.editor.addEventListener("keyup", function(e) {
    var KEY_LEFT = 37, KEY_RIGHT = 39;
    if (e.which == KEY_LEFT || e.which == KEY_RIGHT) {
        RE.enabledEditingItems(e);
    }
});
RE.editor.addEventListener("click", RE.enabledEditingItems);

document.addEventListener('keydown', function(event) {
if (isBreakLineMove(event)) {
if(hasCheckBox()){
 event.preventDefault();
  addCheckBoxAtFirst()
}else{

}
  }
});

function addCheckBoxAtFirst(){
var selection = window.getSelection();
        var range = selection.getRangeAt(0);
        var brNode = document.createElement("br");
        range.insertNode(brNode);

        var checkboxNode = document.createElement("input");
        checkboxNode.type = "checkbox";
        checkboxNode.id = "checkbox";
checkboxNode.onclick = function() { handleClick(this); };
        checkboxNode.classList.add("checkbox");

        brNode.parentNode.insertBefore(checkboxNode, brNode.nextSibling);

        range.setStartAfter(checkboxNode); // set the cursor after the checkbox
        selection.removeAllRanges();
        selection.addRange(range);
}

function isBreakLineMove(event) {
  return (event.keyCode === 13); // Enter key pressed
}

function countBreakLine(){
const breakLineCount = document.querySelectorAll('br').length;
console.log(`There are ${breakLineCount} break lines in this document.`);
console.log(`There are ${getBreakLineIndex()} break lines in this document.`);
}
RE.setTodo = function(text) {
addCheckBox(text)
//if(hasCheckBox()){
//removeCheckboxAtIndex()
//}else{
////addCheckBoxAtFirst()
//    addCheckBox(text)
//}
  //  addCheckBox(text)

}

function addCheckBox(text){
insertCheckBox()
//if(getBreakLineIndex()==0){
// const selection = window.getSelection();
//     const range = selection.getRangeAt(0);
//     const lineElement = range.startContainer.parentNode;
//     lineElement.insertAdjacentHTML('afterbegin', '<input type="checkbox" class="checkbox" onclick="handleClick(this)">');
//}else{
// addCheckboxToBreakLine()
// }
}

function test(){
  //  var html = '<input type="checkbox" class="custom-checkbox"/>&nbsp';
   var html = '<input type="checkbox" class="custom-checkbox">';
     document.execCommand('insertHTML', false, html);
}

function addCheckboxToBreakLine() {
var index = getBreakLineIndex()
  const brElements = document.querySelectorAll('br');
//  console.log(`brElements==${brElements.length}`);
//    console.log(`brElements==${index}`);

  const breakLine = brElements[index-1];

  const checkboxNode = document.createElement("input");
  checkboxNode.type = "checkbox";
    checkboxNode.id = "checkbox";
checkboxNode.onclick = function() { handleClick(this); };
  checkboxNode.classList.add("checkbox");

  if (breakLine) {
      breakLine.parentNode.insertBefore(checkboxNode, breakLine.nextSibling);
    }
}
RE.removeCheckBox = function() {
//let selectedText = window.getSelection().toString();
//console.log(selectedText);

let selObj = window.getSelection();
let range = selObj.getRangeAt(0);
let line = range.startContainer.textContent.split('\n')[range.startOffset];
console.log(line);
//console.log(`There are ${getCheckboxIndex(getBreakLineIndex())} break lines in this document.`);
//console.log(`hasCheckBox:index==${hasCheckBox()}`);
}

function getBreakLineIndex() {
   const brElements = document.querySelectorAll('br');
     const selection = window.getSelection();
     const range = selection.getRangeAt(0);
     const brIndex = Array.from(brElements).findIndex(br => range.startContainer.compareDocumentPosition(br) === Node.DOCUMENT_POSITION_FOLLOWING);
     return brIndex !== -1 ? brIndex : brElements.length;
}

function hasCheckBox() {
var index = getBreakLineIndex()
  const brElements = document.querySelectorAll('br');
  const nextBr = brElements[index];
  const prevBr = brElements[index - 1];
  const range = new Range();
  range.setStartAfter(prevBr || document.body.firstChild);
  range.setEndBefore(nextBr || document.body.lastChild);
  const fragment = range.cloneContents();
  const checkbox = fragment.querySelector('input[type="checkbox"]');
  return !!checkbox;
}

function removeCheckboxAtIndex() {
  var index = getBreakLineIndex()
  var checkboxes = document.querySelectorAll('input[type="checkbox"]');

  console.log(`removeCheckboxAtIndex:index==${index}`);
  console.log(`removeCheckboxAtIndex:checkboxes==${checkboxes.length}`);

  if (checkboxes.length >= index) {
    var checkboxToRemove = checkboxes[index];
    checkboxToRemove.parentNode.removeChild(checkboxToRemove);
  }else{
//removeCheckbox()
  }
}

function log(text){
console.log('TAG==='+text)
}
function removeCheckbox() {
  const selection = window.getSelection();
  const range = selection.getRangeAt(0);
  const checkbox = range.startContainer.parentNode.querySelector('input[type="checkbox"]');

  if (checkbox) {
    const parentNode = checkbox.parentNode;
    parentNode.removeChild(checkbox);
//    range.setStartAfter(parentNode.previousSibling);
//    selection.removeAllRanges();
//    selection.addRange(range);
  }
}

function getCheckboxIndex(breakLineIndex) {
//  const breakLines = document.querySelectorAll('br');
//  const breakLine = breakLines[breakLineIndex];
//  const checkbox = breakLine.previousSibling.querySelector('input[type="checkbox"]');
  const selection = window.getSelection();
    const range = selection.getRangeAt(0);
    const checkbox = range.startContainer.parentNode.querySelector('input[type="checkbox"]');

  if (checkbox) {
    const checkboxes = document.querySelectorAll('input[type="checkbox"]');
    return Array.from(checkboxes).indexOf(checkbox);
  }
  return -1;
}
function insertCheckBox(){
//var html = '<div contenteditable="false" class="list-container"><ul><li><input type="checkbox" onclick="onClickCheckBox(this)" ><textarea placeholder="write your note here..." id="textarea"></textarea><img src="file:///android_asset/cancel_ic.png" alt="Cancel" onclick="removeListItem(this)"></li></ul></div><div contenteditable="false" class="add-container" onclick="addItem()"><img class="add-button" src="file:///android_asset/add_ic.png" alt="Add"><span class="add-text"><u>Add main task</u></span></div>';
//document.execCommand('insertHTML', false, html);
//document.getElementById("textarea1").focus();


const subTitle = document.querySelector('.subTitle');
const newDiv = document.createElement('div');
newDiv.innerHTML = '<div contenteditable="false" class="list-container"><ul><li><input type="checkbox" onclick="onClickCheckBox(this)" ><textarea placeholder="write your note here..." id="textarea"></textarea><img src="file:///android_asset/cancel_ic.png" alt="Cancel" onclick="removeListItem(this)"></li></ul></div><div contenteditable="false" class="add-container" onclick="addItem()"><img class="add-button" src="file:///android_asset/add_ic.png" alt="Add"><span class="add-text"><u>Add main task</u></span></div>';
subTitle.parentNode.insertBefore(newDiv, subTitle.nextSibling);

}

function handleClick(checkbox) {
  if (checkbox.checked) {
const text = checkbox.nextSibling.textContent;
  // get index checkbox
      const container = checkbox.parentElement;
      const checkboxes = container.getElementsByTagName("input");
      const index = Array.prototype.indexOf.call(checkboxes, checkbox);
      console.log("Checkbox " + index + " is checked");

  // get text in line with checkbox

  //setBoldText(text)
//  document.innerHTML.replace(text, `<b>${text}</b>`);
  //document.execCommand('bold', false, null);

  } else {
//setNormal(text)
}
}
RE.insertHeadTitle = function() {
   var html = '<div contenteditable="false" class="title"><textarea placeholder="Note" id="textareaTitle"></textarea></div>';
   document.execCommand('insertHTML', false, html);
}
function removeListItem(cancelButton) {
console.log("cancelButton");
  var listItem = cancelButton.parentNode;
  listItem.parentNode.removeChild(listItem);
}
function addItem(){
const listContainer = document.querySelector('.list-container ul');
 const newItem = document.createElement('li');
    newItem.innerHTML = `<input type="checkbox" onclick="onClickCheckBox(this)"><textarea placeholder="write your note here..." id="textarea"></textarea><img src="file:///android_asset/cancel_ic.png" alt="Cancel" onclick="removeListItem(this)">`;
    listContainer.appendChild(newItem);
}
function onClickCheckBox(checkbox) {
var checkboxes = document.querySelectorAll('input[type="checkbox"]');
var textareas = document.querySelectorAll('textarea');
  var index = Array.prototype.indexOf.call(checkboxes, checkbox);

var textarea = textareas[index];

// console.log(textarea.length)
  if (checkbox.checked) {
      console.log(index);
  textarea.style.textDecoration = 'line-through';
  } else {
    textarea.style.textDecoration = 'none';
        console.log("Checkbox is not checked");
}
}
const textareas = document.querySelectorAll('textarea');

textareas.forEach(function(textarea) {
  textarea.setAttribute("style", "height:" + (textarea.scrollHeight) + "px;overflow-y:hidden;");
  textarea.addEventListener("input", OnInput, false);
});

function OnInput() {
  this.style.height = 0;
  this.style.height = (this.scrollHeight) + "px";
}
function onClickNote(){
console.log("INPUT")
}

window.onload = function() {
console.log("load");
    var textarea1 = document.getElementById("textarea1");
    textarea1.focus();
    textarea1.click();
    textarea1.trigger();

   textarea1.value = "This is the text for textarea1.";
 }



